// TODO: Cleanup imports after refactor
define([
    'dojo/_base/declare',
    'dojo/_base/lang',
    'dojo/_base/window',
    'dojo/dom-construct',
    'dojo/i18n!STM/nls/STMResources',
    'dojo/promise/all',
    'dojo/string',
    'dojo/topic',
    'STM/common/util/ApplicationConstants',
    'STM/common/util/Util',
    'STM/common/view/dialog/DialogTemplate',
    'STM/common/view/dialog/NotificationDialog',
    'STM/dataentry/portlet/model/DataEntryRequests',
    'STM/dataentry/portlet/util/ApplicationConstants',
    'STM/dataentry/portlet/view/DataEntryFormAppView',
    'STM/dataentry/portlet/view/components/DataEntryGridView'
], function(declare, lang, win, domConstruct, resources, all, string, topic, GeneralAppConst, STMUtil, DialogTemplate, 
    NotificationDialog, DataEntryRequests, AppConst, DataEntryFormAppView, DataEntryGridView) {

    var DataEntryPortletApp = declare(null, {

        //
        // Public properties
        //

        // The top-level view
        view: null,
        // Most recent header data
        header: null,

        //
        // Init functions
        //

        startup: function() {
            var me = this;

	    // set initial properties passed from the caller            
            //AppConst.PORTLET_ID = dojoConfig.sas.portletId;
            
            // make sure stm css is included for layout
            //document.body.className = 'stm ' + document.body.className;

            me.listSubscriptions();
            DataEntryRequests.getDataEntryHeader().then(function(header) {
                me.createAppView(header);
            });
        },
        createAppView: function(header) {
            var me = this;
            var state = me._getStateForHeader(header);

            // Set the data entry id which should remain constant for the whole app
            AppConst.DATA_ENTRY_ID = header.form.id;

            me.header = header;

            // Destroy existing app if necessary
            if (me.view) {
                me.view.destroyRecursive();
            }

            me.view = new DataEntryFormAppView();
            domConstruct.place(me.view.domNode, win.body(), 'only');
            me.view.startup();
            STMUtil.setupBusyCursor(me.view.domNode);
            STMUtil.setupTimeout();
            STMUtil.setupShims();

            if (state === AppConst.ENTER_DATA_STATE ||
                state === AppConst.NO_WORKFLOW_STATE ||
                state === AppConst.ADMIN_STATE) {
                DataEntryRequests.getEntryDataForForm(header.startDate, header.scorecards[0].id,
                    header.periodTypes[0].id).then(function(formData) {
                        var props = me._getPropsForEnterState(header, formData);

                        switch(state) {
                            case AppConst.ENTER_DATA_STATE:
                                me.view.createEnterDataView(props);
                                break;
                            case AppConst.NO_WORKFLOW_STATE:
                                me.view.createNoWorkflowView(props);
                                break;
                            case AppConst.ADMIN_STATE:
                                me.view.createAdminView(props);
                                break;
                        }
                    }, function() {
                        new DialogTemplate({
                            type: GeneralAppConst.ERROR_SERVER_ACTION_DIALOG
                        }).show();
                    });
            }
            else if (state === AppConst.APPROVER_READY_STATE) {
                all([
                    DataEntryRequests.getDataEntryUsersForForm(),
                    DataEntryRequests.getAllSubmitterData()
                ]).then(function(results) {
                    me.view.createApproverView(me._getPropsForApproverState(header, results[1], results[0]));
                }, function() {
                    new DialogTemplate({
                        type: GeneralAppConst.ERROR_SERVER_DATA_DIALOG
                    }).show();
                });
            }
            else {
                me.view.createMessageView({
                    message: me._getMessageViewMessage(state)
                });
            }
        },
        listSubscriptions: function() {
            topic.subscribe(AppConst.SAVE_FORM_CLICK, lang.hitch(this, this.onSaveClick));
            topic.subscribe(AppConst.SUBMIT_FORM_CLICK, lang.hitch(this, this.onSubmitClick));
            topic.subscribe(AppConst.DISCARD_CHANGES_CLICK, lang.hitch(this, this.onDiscardClick));
            topic.subscribe(AppConst.SAVELAYOUT_FORM_CLICK, lang.hitch(this, this.onSaveLayoutClick));
            topic.subscribe(AppConst.RESET_FORM_CLICK, lang.hitch(this, this.onResetFormClick));
            topic.subscribe(AppConst.FORMULA_VALIDATION, lang.hitch(this, this.formulaValid));
            topic.subscribe(AppConst.HEADER_CHANGED, lang.hitch(this, this.onHeaderChange));
            topic.subscribe(AppConst.SUBMITTER_CHANGED, lang.hitch(this, this.onSubmitterChange));
            topic.subscribe(AppConst.APPROVE_FORM_CLICK, lang.hitch(this, this.onApproveFormClick));
            topic.subscribe(AppConst.REJECT_FORM_CLICK, lang.hitch(this, this.onRejectFormClick));
            topic.subscribe(AppConst.COMMENTS_CLICK, lang.hitch(this, this.onCommentsClick));
            topic.subscribe(AppConst.LINK_CLICK, lang.hitch(this, this.onLinkClick));
        },

        //
        // Listeners
        //

        onSaveClick: function() {
            var me = this;
            var formData = me.view.getReturnGridData();
            var gridValidState = me.view.isGridValid();
            var dialogPromise;

            // Is it invalid?
            if (gridValidState === DataEntryGridView.INVALID) {
                new DialogTemplate({
                    title: resources['de.portlet.form.datagrid.invalid.title.txt'],
                    type: GeneralAppConst.SIMPLE_CONFIRM_DIALOG,
                    iconType: GeneralAppConst.DIALOG_ERROR_ICON,
                    message: resources['de.portlet.form.datagrid.invalid.save.message.txt']
                }).show();
            }
            else if (gridValidState === DataEntryGridView.MIXED) {
                new DialogTemplate({
                    title: resources['de.portlet.form.datagrid.invalid.title.txt'],
                    type: GeneralAppConst.SIMPLE_CONFIRM_DIALOG,
                    iconType: GeneralAppConst.DIALOG_ERROR_ICON,
                    message: resources['de.portlet.form.datagrid.mixed.save.message.txt']
                }).show();
            }
            // Are there no changes?
            else if (formData.rows.length < 1) {
                new DialogTemplate({
                    title: resources['de.portlet.form.datagrid.no.save.title.txt'],
                    type: GeneralAppConst.SIMPLE_CONFIRM_DIALOG,
                    message: resources['de.portlet.form.datagrid.no.save.message.txt']
                }).show();
            }
            // Ok, save it, but first check for warning rules
            else {
                dialogPromise = new DialogTemplate({
                    title: gridValidState === DataEntryGridView.WARNING ? 
                        resources['de.portlet.form.datagrid.warning.save.title.txt'] : resources['de.portlet.form.datagrid.savedialogtitle.txt'],
                    type: GeneralAppConst.CONFIRM_CANCEL_DIALOG,
                    iconType: gridValidState === DataEntryGridView.WARNING ?
                        GeneralAppConst.DIALOG_WARNING_ICON : GeneralAppConst.DIALOG_INFORMATION_ICON,
                    message: gridValidState === DataEntryGridView.WARNING ?
                        resources['de.portlet.form.datagrid.warning.save.message.txt'] : resources['de.portlet.form.datagrid.savemsg.txt'],
                    confirmLabel: resources['save.txt'],
                    confirmActionAfterClose: true
                }).show();

                dialogPromise.then(function(choice) {
                    if (choice === GeneralAppConst.DIALOG_CONFIRM_ACTION) {
                        DataEntryRequests.saveEntryDataForForm(formData, me.view.getSelectedDate(), me.view.getSelectedScorecard(),
                                me.view.getSelectedPeriodType()).then(function(returnedData) {
                                new NotificationDialog({
                                    message: resources['de.portlet.form.datagrid.save.successful.txt']
                                }).show();
                                me._handleGridUpdate(returnedData);
                            }, function() {
                                new DialogTemplate({
                                    type: GeneralAppConst.ERROR_SERVER_ACTION_DIALOG
                                }).show();
                            });
                    }
                });
            }
        },
        onSubmitClick: function() {
            var me = this;
            var state = me._getStateForHeader(me.header);
            var formData = me.view.getReturnGridData();
            var gridValidState = me.view.isGridValid();
            var dialogPromise, msg;

            // Is it invalid?
            if (gridValidState === DataEntryGridView.INVALID) {
                new DialogTemplate({
                    title: resources['de.portlet.form.datagrid.invalid.title.txt'],
                    type: GeneralAppConst.SIMPLE_CONFIRM_DIALOG,
                    iconType: GeneralAppConst.DIALOG_ERROR_ICON,
                    message: resources['de.portlet.form.datagrid.invalid.submit.message.txt']
                }).show();
            }
            else if (gridValidState === DataEntryGridView.MIXED) {
                new DialogTemplate({
                    title: resources['de.portlet.form.datagrid.invalid.title.txt'],
                    type: GeneralAppConst.SIMPLE_CONFIRM_DIALOG,
                    iconType: GeneralAppConst.DIALOG_ERROR_ICON,
                    message: resources['de.portlet.form.datagrid.mixed.submit.message.txt']
                }).show();
            }
            // Is it a no workflow with no changes?
            else if (formData.rows.length < 1 && state === AppConst.NO_WORKFLOW_STATE) {
                new DialogTemplate({
                    title: resources['de.portlet.form.datagrid.no.submit.title.txt'],
                    type: GeneralAppConst.SIMPLE_CONFIRM_DIALOG,
                    message: resources['de.portlet.form.datagrid.no.submit.message.txt']
                }).show();
            }
            // Ok, submit it, but first check for warning rules
            else {
                if (state === AppConst.NO_WORKFLOW_STATE) {
                    if (gridValidState !== DataEntryGridView.WARNING) {
                        msg = resources['de.portlet.form.datagrid.submitmsg.no.workflow.txt'];
                    }
                    else {
                        msg = resources['de.portlet.form.datagrid.warning.submit.no.workflow.message.txt'];
                    }
                }
                else {
                    if (gridValidState !== DataEntryGridView.WARNING) {
                        msg = resources['de.portlet.form.datagrid.submitmsg.txt'];
                    }
                    else {
                        msg = resources['de.portlet.form.datagrid.warning.submit.message.txt'];
                    }
                }
                
                dialogPromise = new DialogTemplate({
                    title: gridValidState === DataEntryGridView.WARNING ? resources['de.portlet.form.datagrid.warning.submit.title.txt'] : 
                        resources['de.portlet.form.datagrid.submitdialogtitle.txt'],
                    type: GeneralAppConst.CONFIRM_CANCEL_DIALOG,
                    iconType: gridValidState === DataEntryGridView.WARNING ?
                        GeneralAppConst.DIALOG_WARNING_ICON : GeneralAppConst.DIALOG_INFORMATION_ICON,
                    message: msg,
                    confirmLabel: resources['submit.txt'],
                    confirmActionAfterClose: true
                }).show();

                dialogPromise.then(function(choice) {
                    if (choice === GeneralAppConst.DIALOG_CONFIRM_ACTION) {
                        DataEntryRequests.submitEntryDataForForm(formData, me.view.getSelectedDate(), me.view.getSelectedScorecard(),
                                me.view.getSelectedPeriodType()).then(function(returnedData) {
                                if (state !== AppConst.NO_WORKFLOW_STATE) {
                                    // A successful workflow submit will change the state. Need to re-fetch the header and re-create the view
                                    DataEntryRequests.getDataEntryHeader().then(function(header) {
                                        me.createAppView(header);
                                    });
                                }
                                else {
                                    new NotificationDialog({
                                        message: resources['de.portlet.form.datagrid.submit.successful.txt']
                                    }).show();
                                    me._handleGridUpdate(returnedData);
                                }
                            }, function() {
                                new DialogTemplate({
                                    type: GeneralAppConst.ERROR_SERVER_ACTION_DIALOG
                                }).show();
                            });
                    }
                });
            }
        },
        onSaveLayoutClick: function() {
            var me = this;
            var formData = me.view.getSaveWidthGridData();

            DataEntryRequests.saveLayout(formData).then(function() {
                new NotificationDialog({
                    message: resources['de.portlet.form.datagrid.savelayoutmsg.txt']
                }).show();
            }, function() {
                new DialogTemplate({
                    type: GeneralAppConst.ERROR_SERVER_ACTION_DIALOG
                }).show();
            });
        },
        onHeaderChange: function() {
            var me = this;
            var formData = me.view.getReturnGridData();
            var submitting = me._getStateForHeader(me.header) === AppConst.NO_WORKFLOW_STATE;
            var dialogPromise, saveOrSubmitPromise;

            // Is there data to save?
            if (formData.rows.length > 0) {
                dialogPromise = new DialogTemplate({
                    title: submitting ? resources['de.portlet.form.datagrid.unsubmitted.changes.title.txt'] :
                        resources['de.portlet.form.datagrid.unsaved.changes.title.txt'],
                    type: GeneralAppConst.CONFIRM_CANCEL_DIALOG,
                    message: submitting ? resources['de.portlet.form.datagrid.unsubmitted.changes.message.txt'] :
                        resources['de.portlet.form.datagrid.unsaved.changes.message.txt'],
                    confirmLabel: submitting ? resources['de.portlet.form.datagrid.submit.continue.txt'] :
                        resources['de.portlet.form.datagrid.save.continue.txt'],
                    cancelLabel: submitting ? resources['de.portlet.form.datagrid.continue.no.submit.txt'] :
                        resources['de.portlet.form.datagrid.continue.no.save.txt'],
                    confirmActionAfterClose: true
                }).show();

                dialogPromise.then(function(choice) {
                    var attemptDialogPromise, gridValidState;

                    if (choice === GeneralAppConst.DIALOG_CONFIRM_ACTION) {

                        gridValidState = me.view.isGridValid();
                        
                        // Is the grid invalid, or mixed
                        if (gridValidState === DataEntryGridView.INVALID || gridValidState === DataEntryGridView.MIXED) {
                            attemptDialogPromise = new DialogTemplate({
                                title: resources['de.portlet.form.datagrid.invalid.title.txt'],
                                type: GeneralAppConst.CONFIRM_CANCEL_DIALOG,
                                iconType: GeneralAppConst.DIALOG_ERROR_ICON,
                                message: submitting ? resources['de.portlet.form.datagrid.invalid.submit.attempt.message.txt'] :
                                    resources['de.portlet.form.datagrid.invalid.save.attempt.message.txt'],
                                confirmLabel: submitting ? resources['de.portlet.form.datagrid.continue.no.submit.txt'] :
                                    resources['de.portlet.form.datagrid.continue.no.save.txt']
                            }).show();

                            attemptDialogPromise.then(function(choice) {
                                if (choice === GeneralAppConst.DIALOG_CONFIRM_ACTION) {
                                    me._fetchAndUpdateGridData();
                                }
                            });
                        }
                        // Ok, save it
                        else {
                            if (submitting) {
                                saveOrSubmitPromise = DataEntryRequests.submitEntryDataForForm(formData, me.view.getSelectedDate(),
                                    me.view.getSelectedScorecard(), me.view.getSelectedPeriodType());
                            }
                            else {
                                saveOrSubmitPromise = DataEntryRequests.saveEntryDataForForm(formData, me.view.getSelectedDate(),
                                    me.view.getSelectedScorecard(), me.view.getSelectedPeriodType());
                            }

                            saveOrSubmitPromise.then(function(returnedData) {
                                new NotificationDialog({
                                    message: submitting ? resources['de.portlet.form.datagrid.submit.successful.txt'] :
                                        resources['de.portlet.form.datagrid.save.successful.txt']
                                }).show();
                                me._handleGridUpdate(returnedData);
                            }, function() {
                                new DialogTemplate({
                                    type: GeneralAppConst.ERROR_SERVER_ACTION_DIALOG
                                }).show();
                            });
                        }
                    }
                    else {
                        me._fetchAndUpdateGridData();
                    }
                });
            }
            else {
                me._fetchAndUpdateGridData();
            }

        },
        onDiscardClick: function() {
            var me = this;
            var formData = me.view.getReturnGridData();
            var gridValidState = me.view.isGridValid();
            var dialogPromise;

            if (formData.rows.length > 0 || gridValidState === DataEntryGridView.INVALID) {
                dialogPromise = new DialogTemplate({
                    title: resources['de.portlet.form.datagrid.discard.title.txt'],
                    type: GeneralAppConst.CONFIRM_CANCEL_DIALOG,
                    message: resources['de.portlet.form.datagrid.discard.message.txt'],
                    iconType: GeneralAppConst.DIALOG_WARNING_ICON,
                    confirmLabel:resources['de.portlet.form.datagrid.discard.txt'],
                    confirmActionAfterClose: true
                }).show();

                dialogPromise.then(function(choice) {
                    if (choice === GeneralAppConst.DIALOG_CONFIRM_ACTION) {
                        me._fetchAndUpdateGridData().then(function() {
                            new NotificationDialog({
                                message: resources['de.portlet.form.datagrid.discard.success.txt']
                            }).show();
                        });
                    }
                });
            }
            else {
                new DialogTemplate({
                    title: resources['de.portlet.form.datagrid.no.discard.title.txt'],
                    type: GeneralAppConst.SIMPLE_CONFIRM_DIALOG,
                    message: resources['de.portlet.form.datagrid.no.discard.message.txt']
                }).show();
            }
        },
        onResetFormClick: function() {
            var me = this;
            var dialogPromise = new DialogTemplate({
                title: resources['de.portlet.form.datagrid.reset.title.txt'],
                type: GeneralAppConst.CONFIRM_CANCEL_DIALOG,
                message: resources['de.portlet.form.datagrid.reset.message.txt'],
                iconType: GeneralAppConst.DIALOG_WARNING_ICON,
                confirmLabel: resources['de.portlet.form.datagrid.reset.txt'],
                confirmActionAfterClose: true
            }).show();

            dialogPromise.then(function(choice) {
                if (choice === GeneralAppConst.DIALOG_CONFIRM_ACTION) {
                    DataEntryRequests.resetEntryDataFormData(me.view.getReturnGridData(), me.view.getSelectedDate(),
                    me.view.getSelectedScorecard(), me.view.getSelectedPeriodType()).then(function(returnedData) {
                        new NotificationDialog({
                            message: resources['de.portlet.form.datagrid.reset.success.txt']
                        }).show();
                        me._handleGridUpdate(returnedData);
                    }, function() {
                        new DialogTemplate({
                            type: GeneralAppConst.ERROR_SERVER_ACTION_DIALOG
                        }).show();
                    });
                }
            });
        },
        onCommentsClick: function(commentsUrl) {
            window.open(commentsUrl);
        },
        onLinkClick: function(linkUrl, linkWindowFeatures) {
            var dwin = window.open(linkUrl, 'dialogwin', linkWindowFeatures);
            dwin.focus();
        },
        onSubmitterChange: function(newValue) {
            var me = this;
            var getSubmitterDataPromise;

            // Check for "All" selection
            if (newValue === 'all') {
                getSubmitterDataPromise = DataEntryRequests.getAllSubmitterData();
            }
            else {
                getSubmitterDataPromise =
                    DataEntryRequests.getSubmitterData(newValue);
            }

            getSubmitterDataPromise.then(function(formData) {
                me._handleGridUpdate(formData);
            }, function() {
                new DialogTemplate({
                    type: GeneralAppConst.ERROR_SERVER_DATA_DIALOG
                }).show();
            });
        },
        onApproveFormClick: function() {
            var me = this;
            var submitter = me.view.getSelectedSubmitter();
            var isAll = submitter.id === 'all';
            var gridValidState = me.view.isGridValid();
            var dialogPromise;

            // Are there any elements the approver does not have write access to?
            if(gridValidState === DataEntryGridView.INVALID) {
                new DialogTemplate({
                    title: resources['de.portlet.form.datagrid.invalid.approve.title.txt'],
                    type: GeneralAppConst.SIMPLE_CONFIRM_DIALOG,
                    iconType: GeneralAppConst.DIALOG_ERROR_ICON,
                    message: resources['de.portlet.form.datagrid.invalid.approve.message.txt']
                }).show();
            }
            // Ok, approve it
            else {
                dialogPromise = new DialogTemplate({
                    title: isAll ? resources['de.portlet.form.datagrid.approve.all.title.txt'] :
                        string.substitute(resources['de.portlet.form.datagrid.approve.title.txt'], [submitter.name]),
                    type: GeneralAppConst.CONFIRM_CANCEL_DIALOG,
                    message: isAll ? resources['de.portlet.form.datagrid.approve.all.message.txt'] :
                        string.substitute(resources['de.portlet.form.datagrid.approve.message.txt'], [submitter.name]),
                    confirmLabel: resources['de.portlet.form.datagrid.approve.txt']
                }).show();

                dialogPromise.then(function(choice) {
                    var approveSubmitterPromise;
                    if (choice === GeneralAppConst.DIALOG_CONFIRM_ACTION) {
                        approveSubmitterPromise = isAll ?
                            DataEntryRequests.approveAllSubmitters() : DataEntryRequests.approveSubmitter(submitter.id);

                        approveSubmitterPromise.then(function() {
                            // Need to get the header to check for state change
                            DataEntryRequests.getDataEntryHeader().then(function(header) {
                                // State changed
                                if (me._getStateForHeader(me.header) !== me._getStateForHeader(header)) {
                                    me.createAppView(header);
                                }
                                else {
                                    new NotificationDialog({
                                        message: resources['de.portlet.form.datagrid.approve.success.txt']
                                    }).show();

                                    all([
                                        DataEntryRequests.getDataEntryUsersForForm(),
                                        DataEntryRequests.getAllSubmitterData()
                                    ]).then(function(results) {
                                            var props = me._getPropsForApproverState(header, results[1], results[0]);
                                            me.view.updateApproverToolbar(props);
                                            me.view.updateApproverGrid(props);
                                        }, function() {
                                            new DialogTemplate({
                                                type: GeneralAppConst.ERROR_SERVER_DATA_DIALOG
                                            }).show();
                                        });
                                }
                            });
                        }, function() {
                            new DialogTemplate({
                                type: GeneralAppConst.ERROR_SERVER_ACTION_DIALOG
                            }).show();
                        });
                    }
                });
            }
        },
        onRejectFormClick: function() {
            var me = this;
            var submitter = me.view.getSelectedSubmitter();
            var isAll = submitter.id === 'all';
            var dialogPromise;

            dialogPromise = new DialogTemplate({
                title: isAll ? resources['de.portlet.form.datagrid.reject.all.title.txt'] :
                    string.substitute(resources['de.portlet.form.datagrid.reject.title.txt'], [submitter.name]),
                type: GeneralAppConst.CONFIRM_CANCEL_DIALOG,
                message: isAll ? resources['de.portlet.form.datagrid.reject.all.message.txt'] :
                    string.substitute(resources['de.portlet.form.datagrid.reject.message.txt'], [submitter.name]),
                confirmLabel: resources['de.portlet.form.datagrid.reject.txt']
            }).show();

            dialogPromise.then(function(choice) {
                var rejectSubmitterPromise;
                if (choice === GeneralAppConst.DIALOG_CONFIRM_ACTION) {
                    rejectSubmitterPromise = isAll ?
                        DataEntryRequests.rejectAllSubmitters() : DataEntryRequests.rejectSubmitter(submitter.id);

                    rejectSubmitterPromise.then(function() {
                        // Need to get the header to check for state change
                        DataEntryRequests.getDataEntryHeader().then(function(header) {
                            // State changed
                            if (me._getStateForHeader(me.header) !== me._getStateForHeader(header)) {
                                me.createAppView(header);
                            }
                        });
                    }, function() {
                        new DialogTemplate({
                            type: GeneralAppConst.ERROR_SERVER_ACTION_DIALOG
                        }).show();
                    });
                }
            });
        },

        //
        // Private functions
        //

        _getPropsForEnterState: function(header, formData) {
            return {
                date: header.startDate,
                toRange: header.toRange,
                fromRange: header.fromRange,
                scorecards: header.scorecards,
                periodTypes: header.periodTypes,
                project: header.project,
                elementType: header.elementType,
                showLink: header.showLink,
                linkName: header.linkName,
                linkUrl: header.linkUrl,
                linkWindowFeatures: header.linkWindowFeatures,
                dataEntryForm: header.form,
                showProjectName: header.showProjectName,
                dateChangingAllowed: header.dateChangingAllowed,
                commentLinkUrl: header.commentLinkUrl,
                formData: formData
            };
        },
        _getPropsForApproverState: function(header, formData, users) {
            return {
                users: users,
                formData: formData,
                dataEntryForm: header.form,
                commentLinkUrl: header.commentLinkUrl
            };
        },
        // Figure out the state based on the header properties
        _getStateForHeader: function(header) {
            // ** Possible formStatus values (that we have to worry about):
            // * DATA_ENTRY_STATUS_WORKFLOW_IN_PROGRESS(2)
            // * DATA_ENTRY_STATUS_WORKFLOW_PARTIAL_COMPLETE (3)
            // * DATA_ENTRY_STATUS_WORKFLOW_AWAITING_APPROVAL (5)
            // * DATA_ENTRY_STATUS_WORKFLOW_PARTIAL_APPROVED (6)
            // * DATA_ENTRY_STATUS_WORKFLOW_APPROVED (7)

            // ** Possible states for data entry portlet
            // * EDIT_STATE: The portlet admin has yet to set up the portlet
            // * ADMIN_STATE: The portlet admin has created the portlet and checked "admin mode"
            // * NO_WORKFLOW_STATE: Form has been saved and started with No Workflow selected
            // * ENTER_DATA_STATE: Assignee entering data
            //  - formStatus: 2, 3
            //  - userRole: data entry (1) or both (3)
            //  - dataEntrySubmitted: false
            // * SUBMITTED_DATA_STATE: Assignee has already submitted data, not in approval state yet
            //  - formStatus: 3, 5, 6, 7
            //  - userRole: data entry (1) or both (3)
            //  - dataEntrySubmitted: true
            // * AWAITING_APPROVAL_STATE: Approver views portlet before all assignees have submitted
            //  - formStatus: 2, 3
            //  - userRole: approver (2) or both (3)
            // * APPROVER_WAITING_STATE: It isn't the approver's turn to approve
            //  - formStatus: 5, 6
            //  - userRole: approver (2) or both (3)
            //  - currentUserApprovalTurn: false
            // * APPROVER_READY_STATE: It is the approver's turn to approve
            //  - formStatus: 5, 6
            //  - userRole: approver (2) or both (3)
            //  - currentUserApprovalTurn: true
            // * FORM_FINISHED_STATE: Approver views portlet after all approvers have approved
            //  - formStatus: 7
            //  - userRole: approver (2) or both (3)
            // **

            // ** Scenarios where the state can change in mid-flight, aka without page refresh
            // * Assignee submits data successfully, is not an approver
            //  - From ENTER_DATA_STATE to SUBMITTED_DATA_STATE
            // * Assignee submits data successfully (not the last submitter), is also an approver
            //  - From ENTER_DATA_STATE to AWAITING_APPROVAL_STATE
            // * Assignee submits data successfully (is last submitter), is also an approver, is not their turn
            //  - From ENTER_DATA_STATE to APPROVER_WAITING_STATE
            // * Assignee submits data successfully (is last submitter), is also an approver, is their turn
            //  - From ENTER_DATA_STATE to APPROVER_READY_STATE
            // * Approver approves all data, is not last approver
            //  - From APPROVER_READY_STATE to APPROVER_WAITING_STATE
            // * Approver approves all data, is last approver
            //  - From APPROVER_READY_STATE to FORM_FINISHED_STATE
            // * Approver rejects data, does not reject themselves
            //  - From APPROVER_READY_STATE to AWAITING_APPROVAL_STATE
            // * Approver rejects data, rejects themselves
            //  - From APPROVER_READY_STATE to ENTER_DATA_STATE

            var formStatus = header.formStatus;
            var userRole = header.userRole;
            var dataEntrySubmitted = header.dataEntrySubmitted;
            var currentUserApprovalTurn = header.currentUserApprovalTurn;
            var dataEntryCapability = header.dataEntryCapability;
            var approvalCapability = header.approvalCapability;
            var projectAccessible = header.projectAccesible;
            var hasScorecards = STMUtil.isNonEmptyArray(header.scorecards);
            var hasPeriodTypes = STMUtil.isNonEmptyArray(header.periodTypes);

            if (!projectAccessible) {
                return AppConst.NO_PROJECT_READ_ACCESS_STATE;
            }
            else if (header.portletAdminMode && formStatus !== AppConst.DATA_ENTRY_STATUS_DRAFT) {
                if (!hasScorecards) {
                    return AppConst.NO_SCORECARDS_STATE;
                }
                else if (!hasPeriodTypes) {
                    return AppConst.NO_PERIOD_TYPES_STATE;
                }
                else {
                    return AppConst.ADMIN_STATE;
                }
            }
            else if (formStatus === AppConst.DATA_ENTRY_STATUS_NO_WORKFLOW) {
                if (dataEntryCapability) {
                    if (!hasScorecards) {
                        return AppConst.NO_SCORECARDS_STATE;
                    }
                    else if (!hasPeriodTypes) {
                        return AppConst.NO_PERIOD_TYPES_STATE;
                    }
                    else {
                        return AppConst.NO_WORKFLOW_STATE;
                    }
                }
                else {
                    return AppConst.NO_DATA_ENTRY_CAPABILITY_STATE;
                }
            }
            else if (userRole === AppConst.NONE_ROLE) {
                return AppConst.NO_ROLE_STATE;
            }
            else if ((formStatus === AppConst.DATA_ENTRY_STATUS_WORKFLOW_IN_PROGRESS ||
                formStatus === AppConst.DATA_ENTRY_STATUS_WORKFLOW_PARTIAL_COMPLETE) &&
                (userRole === AppConst.DATA_ENTRY_ROLE || userRole === AppConst.BOTH_ROLE) &&
                dataEntrySubmitted === false) {

                if (dataEntryCapability) {
                    if (!hasScorecards) {
                        return AppConst.NO_SCORECARDS_STATE;
                    }
                    else if (!hasPeriodTypes) {
                        return AppConst.NO_PERIOD_TYPES_STATE;
                    }
                    else {
                        return AppConst.ENTER_DATA_STATE;
                    }
                }
                else {
                    return AppConst.NO_DATA_ENTRY_CAPABILITY_STATE;
                }
            }
            else if ((formStatus === AppConst.DATA_ENTRY_STATUS_WORKFLOW_AWAITING_APPROVAL ||
                formStatus === AppConst.DATA_ENTRY_STATUS_WORKFLOW_PARTIAL_APPROVED) &&
                (userRole === AppConst.APPROVER_ROLE || userRole === AppConst.BOTH_ROLE) &&
                currentUserApprovalTurn === true) {

                if (approvalCapability) {
                    return AppConst.APPROVER_READY_STATE;
                }
                else {
                    return AppConst.NO_APPROVAL_CAPABILITY_STATE;
                }
            }
            else if (formStatus === AppConst.DATA_ENTRY_STATUS_DRAFT) {
                return AppConst.DRAFT_STATE;
            }
            else if (formStatus === AppConst.DATA_ENTRY_STATUS_WORKFLOW_NOT_STARTED) {
                return AppConst.NOT_STARTED_STATE;
            }
            else if ((formStatus === AppConst.DATA_ENTRY_STATUS_WORKFLOW_PARTIAL_COMPLETE ||
                formStatus === AppConst.DATA_ENTRY_STATUS_WORKFLOW_AWAITING_APPROVAL ||
                formStatus === AppConst.DATA_ENTRY_STATUS_WORKFLOW_PARTIAL_APPROVED ||
                formStatus === AppConst.DATA_ENTRY_STATUS_WORKFLOW_APPROVED) &&
                userRole === AppConst.DATA_ENTRY_ROLE &&
                dataEntrySubmitted === true) {

                return AppConst.SUBMITTED_DATA_STATE;
            }
            else if ((formStatus === AppConst.DATA_ENTRY_STATUS_WORKFLOW_IN_PROGRESS ||
                formStatus === AppConst.DATA_ENTRY_STATUS_WORKFLOW_PARTIAL_COMPLETE) &&
                (userRole === AppConst.APPROVER_ROLE || userRole === AppConst.BOTH_ROLE)) {

                return AppConst.AWAITING_APPROVAL_STATE;
            }
            else if ((formStatus === AppConst.DATA_ENTRY_STATUS_WORKFLOW_AWAITING_APPROVAL ||
                formStatus === AppConst.DATA_ENTRY_STATUS_WORKFLOW_PARTIAL_APPROVED) &&
                (userRole === AppConst.APPROVER_ROLE || userRole === AppConst.BOTH_ROLE) &&
                currentUserApprovalTurn === false) {

                return AppConst.APPROVER_WAITING_STATE;
            }
            else if ((formStatus === AppConst.DATA_ENTRY_STATUS_WORKFLOW_APPROVED) &&
                (userRole === AppConst.APPROVER_ROLE || userRole === AppConst.BOTH_ROLE)) {

                return AppConst.FORM_FINISHED_STATE;
            }
        },
        _getMessageViewMessage: function(formState) {
            var message;

            switch (formState) {
                case AppConst.DRAFT_STATE:
                    message = resources['de.portlet.form.partial.state.draft.txt'];
                    break;
                case AppConst.NOT_STARTED_STATE:
                    message = resources['de.portlet.form.partial.state.not.started.txt'];
                    break;
                case AppConst.SUBMITTED_DATA_STATE:
                    message = resources['de.portlet.form.partial.state.submitted.txt'];
                    break;
                case AppConst.AWAITING_APPROVAL_STATE:
                    message = resources['de.portlet.form.partial.state.awaiting.txt'];
                    break;
                case AppConst.APPROVER_WAITING_STATE:
                    message = resources['de.portlet.form.partial.state.partial.txt'];
                    break;
                case AppConst.FORM_FINISHED_STATE:
                    message = resources['de.portlet.form.partial.state.complete.txt'];
                    break;
                case AppConst.NO_ROLE_STATE:
                    message = resources['de.portlet.form.partial.state.no.role.txt'];
                    break;
                case AppConst.NO_DATA_ENTRY_CAPABILITY_STATE:
                    message = resources['de.portlet.form.partial.state.no.entry.capability.txt'];
                    break;
                case AppConst.NO_APPROVAL_CAPABILITY_STATE:
                    message = resources['de.portlet.form.partial.state.no.approval.capability.txt'];
                    break;
                case AppConst.NO_PROJECT_READ_ACCESS_STATE:
                    message = resources['de.portlet.form.partial.state.no.project.read.access.txt'];
                    break;
                case AppConst.NO_SCORECARDS_STATE:
                    message = resources['de.portlet.form.partial.state.no.scorecards.txt'];
                    break;
                case AppConst.NO_PERIOD_TYPES_STATE:
                    message = resources['de.portlet.form.partial.state.no.period.types.txt'];
                    break;
            }

            return message;
        },
        _handleGridUpdate: function(formData) {
            var me = this;

            switch(me._getStateForHeader(me.header)) {
                case AppConst.ENTER_DATA_STATE:
                    me.view.updateEnterDataGrid(me._getPropsForEnterState(me.header, formData));
                    break;
                case AppConst.NO_WORKFLOW_STATE:
                    me.view.updateNoWorkflowGrid(me._getPropsForEnterState(me.header, formData));
                    break;
                case AppConst.ADMIN_STATE:
                    me.view.updateAdminGrid(me._getPropsForEnterState(me.header, formData));
                    break;
                case AppConst.APPROVER_READY_STATE:
                    me.view.updateApproverGrid(me._getPropsForApproverState(me.header, formData));
                    break;
            }
        },
        _fetchAndUpdateGridData: function() {
            var me = this;
            var getDataPromise = DataEntryRequests.getEntryDataForForm(me.view.getSelectedDate(), me.view.getSelectedScorecard(),
                    me.view.getSelectedPeriodType());

            getDataPromise.then(function(returnedData) {
                me._handleGridUpdate(returnedData);
            }, function() {
                new DialogTemplate({
                    type: GeneralAppConst.ERROR_SERVER_ACTION_DIALOG
                }).show();
            });

            return getDataPromise;
        }

    });

    return DataEntryPortletApp;
});
