define([
    'dgrid/extensions/ColumnReorder',
    'dgrid/extensions/ColumnResizer',
    'dgrid/extensions/DijitRegistry',
    'dgrid/OnDemandGrid',
    'dijit/form/Button',
    'dijit/form/NumberTextBox',
    'dijit/form/Select',
    'dijit/form/SimpleTextarea',
    'dijit/form/ValidationTextBox',
    'dijit/layout/ContentPane',
    'dijit/registry',
    'dojo/_base/array',
    'dojo/_base/declare',
    'dojo/_base/lang',
    'dojo/aspect',
    'dojo/dom-construct',
    'dojo/dom-geometry',
    'dojo/dom-style',
    'dojo/i18n!STM/nls/STMResources',
    'dojo/keys',
    'dojo/number',
    'dojo/store/Memory',
    'dojo/string',
    'dojo/topic',
    'STM/common/util/ApplicationConstants',
    'STM/common/util/Util',
    'STM/common/view/dialog/DialogTemplate',
    'STM/dataentry/portlet/model/DataEntryRequests',
    'STM/dataentry/portlet/util/ApplicationConstants'
], function(ColumnReorder, ColumnResizer, DijitRegistry, OnDemandGrid, Button, NumberTextBox, Select, SimpleTextarea, 
    ValidationTextBox, ContentPane, registry, array, declare, lang, aspect, domConstruct, domGeometry, domStyle, resources,
    keys, number, Memory, string, topic, GeneralAppConst, STMUtil, DialogTemplate, DataEntryRequests, AppConst) {

    var DataEntryGridView = declare(ContentPane, {

        //
        // Public properties
        //

        _grid: null,
        _returnData: null,
        _noElementsLabel: null,
        _noSubmissionsLabel: null,
        _writeAccessLabel: null,
        _invalidCells: [],
        _warningCells: [],
        _columnWidths: null,

        //
        // Public functions
        //

        createEnterDataGrid: function(props) {
            var me = this;

            me._commonCreateUpdate(
                props,
                me._createEditableGrid,
                me._toggleNoElementsView,
                [
                    me._createSaveButton(),
                    me._createSubmitButton(),
                    me._createDiscardButton()
                ]
            );
            
            me._initializeWidthHandling(props.formData);
        },
        createNoWorkflowGrid: function(props) {
            var me = this;

            me._commonCreateUpdate(
                props,
                me._createEditableGrid,
                me._toggleNoElementsView,
                [
                    me._createSubmitButton(),
                    me._createDiscardButton()
                ]
            );

            me._initializeWidthHandling(props.formData);
        },
        createAdminGrid: function(props) {
            var me = this;

            me._commonCreateUpdate(
                props,
                me._createReadOnlyGrid,
                me._toggleNoElementsView
            );

            me._initializeWidthHandling(props.formData);
        },
        createApproverGrid: function(props) {
            var me = this;

            me._commonCreateUpdate(
                props,
                me._createApproverGrid,
                me._toggleNoSubmissionsView,
                [
                    me._createApproveButton(),
                    me._createRejectButton()
                ]
            );

            me._toggleWriteAccessMessage(array.some(props.formData.rows, function(row) {
                return !row.writable;
            }));
        },
        updateEnterDataGrid: function(props) {
            var me = this;

            me._invalidCells = [];
            me._warningCells = [];
            me._commonCreateUpdate(props, me._createEditableGrid, me._toggleNoElementsView);
            me._initializeWidthHandling(props.formData);
        },
        updateNoWorkflowGrid: function(props) {
            var me = this;

            me._invalidCells = [];
            me._warningCells = [];
            me._commonCreateUpdate(props, me._createEditableGrid, me._toggleNoElementsView);
            me._initializeWidthHandling(props.formData);
        },
        updateAdminGrid: function(props) {
            var me = this;

            me._commonCreateUpdate(props, me._createReadOnlyGrid, me._toggleNoElementsView);
            me._initializeWidthHandling(props.formData);
        },
        updateApproverGrid: function(props) {
            var me = this;

            me._commonCreateUpdate(props, me._createApproverGrid);
            me._toggleWriteAccessMessage(array.some(props.formData.rows, function(row) {
                return !row.writable;
            }));
        },
        getReturnData: function() {
            var me = this;

            return me._returnData;
        },
        getSaveWidthGridData: function() {
            var me = this;
            
            // Mix the current column widths into the return data
            if (me._columnWidths) {
                me._returnData.elementColumnWidth = me._columnWidths.elementColumnWidth;
                
                array.forEach(me._returnData.columns, function(column) {
                    column.columnWidth = me._columnWidths[column.columnId];
                });
            }
            
            return me._returnData;
        },
        // Functions for both invalid cells, and elements with no write access in approver view
        isGridValid: function() {
            var me = this;
            var validState;
            
            if (me._writeAccessLabel) {
                return DataEntryGridView.INVALID;
            }
            else {
                array.forEach(me._grid.store.data, function(item) {
                    var index, column, cell, widgets, widget;
                    
                    for (index in me._grid.columns)  {
                        if (me._grid.columns.hasOwnProperty(index)) {
                            column = me._grid.columns[index];
                            cell = me._grid.cell(item, column.id);
                            widgets = registry.findWidgets(cell.element);
                            
                            if (STMUtil.isNonEmptyArray(widgets)) {
                                widget = widgets[0];
                                
                                if (widget instanceof ValidationTextBox || widget instanceof Select) {
                                    // For required empty cells, force the validation, otherwise cells that haven't been interacted
                                    // with yet can behave strangely
                                    if (widget.get('required') && STMUtil.isEmpty(widget.get('value'))) {
                                        // Text vs choice vs numeric cells have to be handled differently
                                        if (widget instanceof  NumberTextBox) {
                                            STMUtil.forceValidationTextBoxOnce(widget, false);
                                            STMUtil.validateAndHighlight(widget);
                                        }
                                        else if (widget instanceof Select) {
                                            widget._hasBeenBlurred = true;
                                            widget.validate();
                                        }
                                        else {
                                            STMUtil.forceValidationTextBox(widget, false);
                                            STMUtil.validateAndHighlight(widget);
                                        }
                                    }
                                    else {
                                        widget.validate();
                                    }
                                }
                            }
                        }
                    }
                });
            }

            if (me._invalidCells.length > 0 && me._warningCells.length > 0) {
                validState = DataEntryGridView.MIXED;
            }
            else if (me._invalidCells.length > 0) {
                validState = DataEntryGridView.INVALID;
            }
            else if (me._warningCells.length > 0) {
                validState = DataEntryGridView.WARNING;
            }
            else {
                validState = DataEntryGridView.VALID;
            }
            
            return validState;
        },

        //
        // Private functions
        //

        _commonCreateUpdate: function(props, createGridFn, toggleEmptyViewFn, buttons) {
            var me = this;

            me._setReturnData(props.formData);

            if (STMUtil.isNonEmptyArray(props.formData.rows)) {
                me._createAndPlaceGrid(props.formData, createGridFn);
            }

            array.forEach(buttons, function(button) {
                me.addChild(button);
            });

            if (toggleEmptyViewFn) {
                toggleEmptyViewFn.call(me, !STMUtil.isNonEmptyArray(props.formData.rows));
            }
        },
        _initializeWidthHandling: function(formData) {
            var me = this;
            
            if (me._grid) {
                // Initialize _columnWidths, should only happen on the first create
                if (!me._columnWidths) {
                    me._columnWidths = {
                        elementColumnWidth: formData.elementColumnWidth
                    };
                    
                    array.forEach(formData.columns, function(column) {
                        me._columnWidths[column.columnId] = column.columnWidth;
                    });
                }

                me._grid.on('dgrid-columnresize', function(event) {
                    var columnId = event.grid.column(event.columnId).columnLookupId;
                    var newWidth = event.width.toString();

                    if (columnId === 'elementName') {
                        me._columnWidths.elementColumnWidth = newWidth;
                    }
                    else {
                        me._columnWidths[columnId] = newWidth;
                    }
                });
            }
        },
        _setReturnData: function(formData) {
            var me = this;

            me._returnData = lang.clone(formData);
            me._returnData.rows = [];
        },
        _createAndPlaceGrid: function(formData, createGridFn) {
            var me = this;

            if (me._grid) {
                me._grid.destroyRecursive();
            }

            me._grid = createGridFn.call(me, formData);
            domConstruct.place(me._grid.domNode, me.domNode, 'first');
            me._grid.startup();
        },
        _createEditableGrid: function(formData) {
            var me = this;
            var store = new Memory({
                data: formData.rows
            });
            
            // Use the elementId as the row identifier
            store.getIdentity = function(object) {
                return object.elementId;
            };
            
            var grid = new DataEntryGridView._EditableDgrid({
                'class': 'editableDgrid',
                columns: me._createEditableGridColumns(formData.columns, formData.rules, formData.elementColumnWidth),
                store: store,
                bufferRows: 10000
            });

            return grid;
        },
        _createEditableGridColumns: function(columnData, ruleData, elementColumnWidth) {
            var me = this;
            var columns = array.map(columnData, function(column) {
                var colWidth;
                
                if (me._columnWidths) {
                    colWidth = me._columnWidths[column.columnId];
                }
                else {
                    colWidth = column.columnWidth;
                }
                
                return {
                    field: column.columnId + 'value',
                    columnLookupId: column.columnId,
                    renderHeaderCell: function(node) {
                        node.innerHTML = '<b>' + column.columnName + '<br>' + column.periodName + '<br>' +
                            (column.columnType === AppConst.COLUMN_VALUE ?
                            resources['de.portlet.form.datagrid.numeric'] : resources['de.portlet.form.datagrid.text']) + '</b>';
                    },
                    renderCell: function(row, value, node, options) {
                        var rule = me._getRuleForCell(column, row, ruleData);
                        var cellProps = {
                            required: rule && rule.requiredEntry
                        };
                        var numericFormatter = function(value) {
                            return value ? value.toString() : '';
                        };
                        var cellWidget, skipHandle, i, keyValueChoices, delimiter, choices;
                        
                        if (cellProps.required) {
                            cellProps.placeHolder = resources['de.portlet.form.datagrid.requiredfield.txt'];
                        }

                        // Numeric cell
                        if (column.columnType === AppConst.COLUMN_VALUE && (!rule || rule.ruleType === AppConst.NUMERIC_RULE ||
                            rule.ruleType === AppConst.FORMULA_RULE)) {
                            if (column.editable && row.writable) {

                                domStyle.set(node, 'padding', '0px');
                                cellProps['class'] = 'dgridNumberCellWidget';

                                if (rule) {
                                    switch(rule.ruleType) {
                                        case AppConst.NUMERIC_RULE:
                                            cellProps.constraints = {
                                                min: rule.lowRange,
                                                max: rule.highRange,
                                                places: "0,15"
                                            };
                                            cellProps.invalidMessage = resources['de.portlet.form.datagrid.numberinput.txt'];
                                            cellProps.rangeMessage = string.substitute(
                                                resources['de.portlet.form.datagrid.numberoutrangeError.txt'], [rule.lowRange , rule.highRange]);
                                            cellWidget = new NumberTextBox(cellProps);

                                            // Hack to ensure validate doesn't run when we initially set the value
                                            skipHandle = aspect.around(cellWidget, 'validate', function() {
                                                return function() {
                                                    if (skipHandle) {
                                                        skipHandle.remove();
                                                    }
                                                }
                                            });

                                            cellWidget.set('value', parseFloat(value));

                                            me._setCellWidgetChangeListener(cellWidget, row.elementId, column.columnId, 
                                                numericFormatter, rule.allowOverride);
                                            break;
                                        case AppConst.FORMULA_RULE:
                                            cellProps.invalidMessage = rule.formulaMsg;
                                            cellWidget = new NumberTextBox(cellProps);

                                            STMUtil.forceValidationTextBox(cellWidget, true);
                                            cellWidget.set('value', parseFloat(value));

                                            me._setFormulaCellWidgetChangeListener(cellWidget, row.elementId, column.columnId,
                                                rule.id, column.periodId, numericFormatter, rule.allowOverride);
                                            break;
                                    }

                                    me._setCellWidgetValidateListener(cellWidget, row.elementId, column.columnId, rule.allowOverride);
                                }
                                else {
                                    cellProps.constraints = {
                                        places: "0,15"
                                    };
                                    cellProps.value = parseFloat(value);
                                    cellWidget = new NumberTextBox(cellProps);
                                    me._setCellWidgetChangeListener(cellWidget, row.elementId, column.columnId, numericFormatter);
                                }

                                cellWidget._destroyOnRemove = true;

                                // Force to 0 if spinner is clicked while there is no value, rather than the default max or min
                                cellWidget.adjust = function(val, delta) {
                                    if (isNaN(val)) {
                                        return 0;
                                    }
                                    else {
                                        return cellWidget.__proto__.adjust(val, delta);
                                    }
                                };

                                return cellWidget.domNode;
                            }
                            else {
                                // Avoid formatting a blank string into a "0" string
                                if (!STMUtil.isBlank(value)) {
                                    value = number.format(value, { places: "0,15" });
                                }
                            }
                        }
                        // Text cell
                        else if (column.columnType === AppConst.COLUMN_TEXT &&
                            (!rule || rule.ruleType === AppConst.TEXT_RULE || rule.ruleType === AppConst.FORMULA_RULE)) {
                            if (column.editable && row.writable) {

                                domStyle.set(node, 'padding', '0px');
                                cellProps['class'] = 'dgridTextCellWidget';
                                cellProps.value = value;
                                cellWidget = new ValidationTextBox(cellProps);
                                cellWidget._destroyOnRemove = true;

                                // Make only the outer, non-input div of the widget focusable
                                cellWidget.domNode.tabIndex = 0;
                                cellWidget.focusNode.tabIndex = -1;

                                if (cellProps.required || (rule && rule.ruleType === AppConst.FORMULA_RULE)) {
                                    STMUtil.forceValidationTextBox(cellWidget, true);
                                    me._setCellWidgetValidateListener(cellWidget, row.elementId, column.columnId, rule.allowOverride);
                                }

                                cellWidget.on('click, keyup', function(event) {
                                    if (event.type === 'click' || (event.type === 'keyup' && event.keyCode === keys.ENTER)) {
                                        var entryFieldProps = {};
                                        var dialog, dialogPromise, entryField;

                                        entryFieldProps.value = cellWidget.get('value');
                                        entryFieldProps.rows = 15;
                                        entryFieldProps.cols = 65;
                                        entryFieldProps.style = 'width: auto;'

                                        if (!rule) {
                                            entryFieldProps.maxLength = 65535;
                                        }
                                        else if (rule.ruleType === AppConst.FORMULA_RULE) {
                                            entryFieldProps.maxLength = 500;
                                        }
                                        else if (rule.ruleType === AppConst.TEXT_RULE) {
                                            entryFieldProps.maxLength = rule.maxLength < 65535 ? rule.maxLength : 65535;
                                        }

                                        entryField = new SimpleTextarea(entryFieldProps);
                                        entryField.startup();
                                        
                                        
                                        dialog = new DialogTemplate({
                                            title: resources['de.portlet.form.datagrid.edit.text.title.txt'],
                                            type: GeneralAppConst.CONFIRM_CANCEL_DIALOG,
                                            iconType: GeneralAppConst.DIALOG_NO_ICON,
                                            actionsBeforeDestroy: true,
                                            content: entryField
                                        });

                                        dialog.on('show', function() {
                                            entryField.domNode.focus();
                                            STMUtil.moveCursorToEnd(entryField.domNode);
                                        });

                                        dialogPromise = dialog.show();

                                        dialogPromise.then(function(choice) {
                                            var entryValue = entryField.get('value');
                                            if (choice === GeneralAppConst.DIALOG_CONFIRM_ACTION) {

                                                if (entryValue !== cellWidget.get('value')) {
                                                    cellWidget.set('value', entryValue);

                                                    if (rule && rule.ruleType === AppConst.FORMULA_RULE) {
                                                        // Empty strings break the current REST API... that's probably bad
                                                        // design, for now just mark it invalid
                                                        if (STMUtil.isEmpty(entryValue)) {
                                                            if (rule.allowOverride) {
                                                                me._manageValueChange(entryValue, row.elementId, column.columnId);
                                                            }
                                                            STMUtil.forceValidationTextBox(cellWidget, false);
                                                        }
                                                        else {
                                                            // Validate
                                                            DataEntryRequests.validateRule(rule.id, entryValue, row.elementId,
                                                                column.columnId, column.periodId).then(function(isValid) {
                                                                    STMUtil.forceValidationTextBox(cellWidget, isValid && isValid !== 'false');
                                                                    if ((isValid && isValid !== 'false') || rule.allowOverride) {
                                                                        me._manageValueChange(entryValue, row.elementId, column.columnId);
                                                                    }
                                                                }, function() {
                                                                    new DialogTemplate({
                                                                        type: GeneralAppConst.ERROR_SERVER_ACTION_DIALOG
                                                                    }).show();
                                                                });

                                                        }
                                                    }
                                                    else {
                                                        if (!cellProps.required || !STMUtil.isEmpty(entryValue) || rule.allowOverride) {
                                                            me._manageValueChange(entryValue, row.elementId, column.columnId);
                                                        }
                                                    }
                                                }

                                                aspect.after(dialog, 'destroy', function() {
                                                    entryField.destroy();
                                                });
                                            }

                                            // For required cells, validate the cell if
                                            // a) We're confirming, or
                                            // b) We're cancelling, but the cell value started off empty
                                            if (cellProps.required && ((choice === GeneralAppConst.DIALOG_CONFIRM_ACTION) || 
                                                (choice === GeneralAppConst.DIALOG_CANCEL_ACTION && STMUtil.isEmpty(cellWidget.get('value'))))) {
                                                STMUtil.forceValidationTextBox(cellWidget, !STMUtil.isEmpty(entryValue));
                                            }

                                            aspect.after(dialog, 'destroy', function() {
                                                cellWidget.domNode.focus();
                                            })
                                        });
                                    }
                                });

                                return cellWidget.domNode;
                            }
                        }
                        // Choice cell
                        else if (column.columnType === AppConst.COLUMN_TEXT && (rule && rule.ruleType === AppConst.CHOICES_RULE)) {
                            if (column.editable && row.writable) {

                                i = 0;
                                keyValueChoices = cellProps.required && STMUtil.isEmpty(value) ?
                                [
                                    {
                                        value: '',
                                        label: '<span class="defaultDropdownChoice">' +
                                            resources['de.portlet.form.datagrid.requiredfield.txt'] + '</span>'
                                    }
                                ] :
                                [
                                    {
                                        value: '',
                                        label: '<span class="defaultDropdownChoice">' +
                                            resources['de.portlet.form.datagrid.select.choice.txt'] + '</span>'
                                    }
                                ];

                                domStyle.set(node, 'padding', '0px');
                                cellProps['class'] = 'dgridChoiceCellWidget';

                                // Parse on semicolon, or either of the UNICODE semicolons
                                if (rule.choiceList.indexOf(';') > -1)
                                    delimiter = ";";
                                else if (rule.choiceList.indexOf('\u003B') > -1)
                                    delimiter = '\u003B';
                                else if (rule.choiceList.indexOf('\uFF1B') > -1)
                                    delimiter = '\uFF1B';
                                else
                                    delimiter = ";";

                                choices = rule.choiceList.split(delimiter);

                                // Convert choices string to proper key/value array expected by Select
                                if (choices.length && choices.length > 0) {
                                    for (i; i < choices.length; i++) {
                                        keyValueChoices.push({
                                            value: choices[i],
                                            label: choices[i]
                                        });
                                    }
                                }

                                cellProps.value = value;
                                cellProps.options = keyValueChoices;

                                cellWidget = new Select(cellProps);
                                cellWidget._destroyOnRemove = true;
                                me._setCellWidgetChangeListener(cellWidget, row.elementId, column.columnId, function(newValue) {
                                    return newValue || '';
                                }, rule.allowOverride);

                                if (cellProps.required && STMUtil.isEmpty(value)) {
                                    skipHandle = aspect.before(cellWidget, 'loadAndOpenDropDown', function() {
                                        // Remove the default option the first time the dropdown is shown for a required choice rule
                                        keyValueChoices.splice(0, 1);
                                        // Remove the handle so this logic only runs once
                                        skipHandle.remove();
                                        // Required to make it load after we've removed the default option when using the arrow keys
                                        cellWidget._isLoaded = false;
                                        
                                        // Need to store the new value here, or the usual focus/blur listener will not detect
                                        // a new value when selecting the first choice
                                        me._manageValueChange(keyValueChoices[0].value, row.elementId, column.columnId);
                                    });
                                }
                                
                                if (cellProps.required) {
                                    me._setCellWidgetValidateListener(cellWidget, row.elementId, column.columnId, rule.allowOverride);
                                }

                                return cellWidget.domNode;
                            }
                        }

                        domStyle.set(node, 'color', '#949eac');
                        STMUtil.enableOverflowTooltipsForNode(node);
                        node.innerHTML = value;
                    },
                    width: colWidth !== '-1' ? parseInt(colWidth) : null
                };
            });

            if (me._columnWidths) {
                elementColumnWidth = me._columnWidths.elementColumnWidth;
            }
            
            // Stick the element column on the front
            columns.splice(0, 0, {
                field: 'elementName',
                columnLookupId: 'elementName',
                renderHeaderCell: function(node) {
                    node.innerHTML = '<b>' + resources['de.portlet.form.datagrid.elementname.txt'] + '</b>';
                },
                width: elementColumnWidth !== '-1' ? parseInt(elementColumnWidth) : null
            });

            return columns;
        },
        _createReadOnlyGrid: function(formData) {
            var me = this;
            return new DataEntryGridView._ReadOnlyDgrid({
                'class': 'readOnlyDgrid',
                columns: me._createReadOnlyGridColumns(formData.columns, formData.elementColumnWidth),
                store: new Memory({
                    data: formData.rows
                })
            });
        },
        _createReadOnlyGridColumns: function(columnData, elementColumnWidth) {
            var me = this;
            var columns = array.map(columnData, function(column) {
                var colWidth;

                if (me._columnWidths) {
                    colWidth = me._columnWidths[column.columnId];
                }
                else {
                    colWidth = column.columnWidth;
                }
                
                return {
                    field: column.columnId + 'value',
                    columnLookupId: column.columnId,
                    renderHeaderCell: function(node) {
                        node.innerHTML = '<b>' + column.columnName + '<br>' + column.periodName + '<br>' +
                            (column.columnType === AppConst.COLUMN_VALUE ?
                                resources['de.portlet.form.datagrid.numeric'] : resources['de.portlet.form.datagrid.text']) + '</b>';
                    },
                    renderCell: function(row, value, node, options) {
                        node.innerHTML = column.columnType === AppConst.COLUMN_VALUE ? number.format(value, { places: "0,15" }) : value;
                    },
                    width: colWidth !== '-1' ? parseInt(colWidth) : null
                };
            });

            if (me._columnWidths) {
                elementColumnWidth = me._columnWidths.elementColumnWidth;
            }
            
            // Stick the element column on the front
            columns.splice(0, 0, {
                field: 'elementName',
                columnLookupId: 'elementName',
                renderHeaderCell: function(node) {
                    node.innerHTML = '<b>' + resources['de.portlet.form.datagrid.elementname.txt'] + '</b>';
                },
                width: elementColumnWidth !== '-1' ? parseInt(elementColumnWidth) : null
            });

            return columns;
        },
        _createApproverGrid: function(formData) {
            var me = this;
            var grid = new DataEntryGridView._ApproverGrid({
                'class': 'approverDgrid',
                columns: me._createApproverGridColumns(formData.columns),
                store: new Memory({
                    data: formData.rows
                })
            });

            return grid;
        },
        _createApproverGridColumns: function(columnData) {
            var columns = [];

            columns.push({
                field: 'elementName',
                label: resources['de.portlet.form.datagrid.elementname.txt'],
                renderCell: function(row, value, node, options) {
                    if (!row.writable) {
                        node.innerHTML = domGeometry.isBodyLtr() ? '*' + value : value + '*';
                    }
                    else {
                        node.innerHTML = value;
                    }
                }
            }, {
                field: 'userid',
                label: resources['de.portlet.form.datagrid.modify.txt']
            }, {
                field: 'scorecardName',
                label: resources['de.portlet.form.datagrid.scorecard.txt']
            }, {
                field: 'periodName',
                label: resources['de.portlet.form.datagrid.period.txt']
            });

            array.forEach(columnData, function(column) {
                columns.push({
                    field: column.columnId + 'value',
                    renderHeaderCell: function(node) {
                        node.innerHTML = '<b>' + column.columnName + '<br>' + (column.columnType === AppConst.COLUMN_VALUE ?
                            resources['de.portlet.form.datagrid.numeric'] : resources['de.portlet.form.datagrid.text']) + '</b>';
                    },
                    renderCell: function(row, value, node, options) {
                        var displayValue;

                        if (value) {
                            displayValue = column.columnType === AppConst.COLUMN_VALUE ? number.format(value, { places: "0,15" }) : value;
                        }
                        else {
                            displayValue = '';
                        }

                        node.innerHTML = displayValue;
                    }
                });
            });

            return columns;
        },
        _getRuleForCell: function(column, row, rules) {
            var ruleId = row[column.columnId + 'rule'];
            var i = 0;
            for (i; i < rules.length; i++) {
                if (rules[i].id === ruleId) {
                    return rules[i];
                }
            }

            return null;
        },
        _setCellWidgetChangeListener: function(widget, elementId, columnId, formatterFn, isWarning) {
            var me = this;

            widget.on('focus', function() {
                var focusValue = widget.get('value');
                var blurHandle = widget.on('blur', function() {
                    var blurValue = widget.get('value');

                    if (!STMUtil.equalNaN(blurValue, focusValue)) {
                        if (widget.isValid() || isWarning) {
                            // Save change
                            me._manageValueChange(formatterFn ? formatterFn(blurValue) : blurValue, elementId, columnId);
                        }
                    }

                    blurHandle.remove();
                });
            });
        },
        _setFormulaCellWidgetChangeListener: function(widget, elementId, columnId, ruleId, periodId, formatterFn, isWarning) {
            var me = this;

            widget.on('focus', function() {
                var focusValue = widget.get('value');
                var blurHandle = widget.on('blur', function() {
                    var blurValue = widget.get('value');

                    if (!STMUtil.equalNaN(blurValue, focusValue)) {
                        DataEntryRequests.validateRule(ruleId, blurValue, elementId, columnId, periodId).then(function(isValid) {
                            // Dojo request has problems returning false as a boolean
                            if (isValid && isValid !== 'false') {
                                STMUtil.forceValidationTextBox(widget, true);
                                // Save change
                                me._manageValueChange(formatterFn ? formatterFn(blurValue) : blurValue, elementId, columnId);
                            }
                            else {
                                if (isWarning) {
                                    me._manageValueChange(formatterFn ? formatterFn(blurValue) : blurValue, elementId, columnId);
                                }
                                
                                STMUtil.forceValidationTextBox(widget, false);
                            }

                        }, function() {
                            new DialogTemplate({
                                type: GeneralAppConst.ERROR_SERVER_ACTION_DIALOG
                            }).show();
                        });
                    }

                    blurHandle.remove();
                });
            });
        },
        _setCellWidgetValidateListener: function(widget, elementId, columnId, isWarning) {
            var me = this;

            aspect.after(widget, 'validate', function(isValid) {
                me._manageInvalidCells(isValid, elementId, columnId, isWarning);
                return isValid;
            });
        },
        _manageValueChange: function(newValue, elementId, columnId) {
            var me = this;
            var rows = me._returnData.rows;
            var i, len, foundRow, newRow;

            for (i = 0, len = rows.length; i < len; i++) {
                if (rows[i].elementId === elementId) {
                    foundRow = rows[i];
                    break;
                }
            }

            if (foundRow) {
                foundRow[columnId + AppConst.COLUMN_VALUE] = newValue;
            }
            else {
                newRow = {};
                newRow.elementId = elementId;
                newRow[columnId + AppConst.COLUMN_VALUE] = newValue;
                rows.push(newRow);
            }

            // Extending the session here should cover all changes to cell values
            STMUtil.extendPortalSession();
        },
        _manageInvalidCells: function(isValid, elementId, columnId, isWarning) {
            var me = this;
            var invalidList = isWarning ? me._warningCells : me._invalidCells;
            var index = array.indexOf(invalidList, elementId + columnId);

            if (!isValid) {
                if (index < 0) {
                    invalidList.push(elementId + columnId);
                }
            }
            else {
                if (index >= 0) {
                    invalidList.splice(index, 1);
                }
            }
        },
        _createSaveButton: function() {
            var me = this;
            var saveButton = new Button({
                label: resources['save.txt'],
                baseClass: 'dijitButton spacedLeadingButton'
            });

            me.own(
                saveButton.on('click', function() {
                    topic.publish(AppConst.SAVE_FORM_CLICK);
                })
            );

            return saveButton;
        },
        _createSubmitButton: function() {
            var me = this;
            var submitButton = new Button({
                label: resources['de.portlet.form.datagrid.submit.txt'],
                baseClass: 'dijitButton spacedLeadingButton'
            });

            me.own(
                submitButton.on('click', function() {
                    topic.publish(AppConst.SUBMIT_FORM_CLICK);
                })
            );

            return submitButton;
        },
        _createDiscardButton: function() {
            var me = this;
            var discardButton = new Button({
                label: resources['de.portlet.form.datagrid.discard.txt']
            });

            me.own(
                discardButton.on('click', function() {
                    topic.publish(AppConst.DISCARD_CHANGES_CLICK);
                })
            );

            return discardButton;
        },
        _createApproveButton: function() {
            var me = this;
            var approveButton = new Button({
                label: resources['de.portlet.form.datagrid.approve.txt'],
                baseClass: 'dijitButton spacedLeadingButton'
            });

            me.own(
                approveButton.on('click', function() {
                    topic.publish(AppConst.APPROVE_FORM_CLICK);
                })
            );

            return approveButton;
        },
        _createRejectButton: function() {
            var me = this;
            var rejectButton = new Button({
                label: resources['de.portlet.form.datagrid.reject.txt']
            });

            me.own(
                rejectButton.on('click', function() {
                    topic.publish(AppConst.REJECT_FORM_CLICK);
                })
            );

            return rejectButton;
        },
        _toggleNoElementsView: function(show) {
            var me = this;

            if (show && !me._noElementsLabel) {
                // Hide existing content
                STMUtil.hideImmediateChildren(me.domNode);

                // Add the message
                me._noElementsLabel = STMUtil.createAndPlaceCenteredTextNode(
                    resources['de.portlet.form.datagrid.no.elements.message.txt'],
                    me.domNode
                );
            }
            else if (!show && me._noElementsLabel) {
                // Remove the message
                domConstruct.destroy(me._noElementsLabel);
                me._noElementsLabel = null;

                // Show existing content
                STMUtil.showImmediateChildren(me.domNode);
            }
        },
        _toggleNoSubmissionsView: function(show) {
            var me = this;

            // It actually may not be currently possible to reach the toggle OFF code. As designed, the submitter list
            // filters out submitters who submitted no data, while if ALL submitters submitted not data, you will still
            // see the (all) view. This is currently the only view we have to worry about.
            if (show && !me._noSubmissionsLabel) {
                me._noSubmissionsLabel =
                    domConstruct.place(STMUtil.getCenteredTextNode(resources['de.portlet.form.datagrid.no.submissions.txt'],
                        '400px'), me.domNode, 'first');
            }
            else if(!show && me._noElementsLabel) {
                domConstruct.destroy(me._noSubmissionsLabel);
                me._noSubmissionsLabel = null;
            }
        },
        _toggleWriteAccessMessage: function(show) {
            var me = this;

            if (show && !me._writeAccessLabel) {
                me._writeAccessLabel = domConstruct.place('<p style="display: inline-block">' +
                    resources['de.portlet.form.datagrid.no.write.access.message.txt'] +
                    '</p>',
                me.domNode);
            }
            else if (!show && me._writeAccessLabel) {
                domConstruct.destroy(me._writeAccessLabel);
                me._writeAccessLabel = null;
            }
        }

    });

    //
    // Statics
    //

    DataEntryGridView._EditableDgrid = declare([OnDemandGrid, DijitRegistry, ColumnResizer, ColumnReorder]);
    DataEntryGridView._ReadOnlyDgrid = declare([OnDemandGrid, DijitRegistry, ColumnResizer]);
    DataEntryGridView._ApproverGrid = declare([OnDemandGrid, DijitRegistry, ColumnResizer, ColumnReorder]);
    
    // Possible grid validation states
    DataEntryGridView.VALID = 'valid';
    DataEntryGridView.WARNING = 'warning';
    DataEntryGridView.INVALID = 'invalid';
    DataEntryGridView.MIXED = 'mixed';

    return DataEntryGridView;

});
