define([
    'dojo/i18n!STM/nls/STMResources',
    'dojo/_base/declare',
    'dojo/_base/array',
    'dojo/string',
    'dojo/dom',
    'dojo/dom-construct',
    'dojo/dom-style',
    'dojo/store/Memory',
    'dojo/topic',
    'dojo/when',
    'dijit/Toolbar',
    'dijit/ToolbarSeparator',
    'dijit/registry',
    'dijit/form/Button',
    'STM/dataentry/portlet/util/ApplicationConstants',
    'STM/common/view/STMFilteringSelect',
    'STM/common/util/Util'
], function(resources, declare, array, string, dom, domConstruct, domStyle, Memory, topic, when, Toolbar, ToolbarSeparator,
            registry, Button, AppConst, STMFilteringSelect, STMUtil) {

    var DataEntryToolbarView = declare(Toolbar, {

        //
        // Public properties
        //

        _currentlySelectedSubmitter: null,

        //
        // Public functions
        //

        createEnterDataToolbar: function(props) {
            var me = this;

            me.addChild(me._createSaveDataButton());
            me.addChild(me._createSubmitDataButton());
            me.addChild(new ToolbarSeparator());
            me.addChild(me._createCommentsButton(props.commentLinkUrl));
            me.addChild(new ToolbarSeparator());
            me.addChild(me._createResetFormButton());
            me._createAndPlaceFormLabel(props.dataEntryForm.name);
            me._calculateAndSetMinWidth(me.domNode);
        },
        createNoWorkflowToolbar: function(props) {
            var me = this;

            me.addChild(me._createSubmitDataButton());
            me.addChild(new ToolbarSeparator());
            me.addChild(me._createCommentsButton(props.commentLinkUrl));
            me._createAndPlaceFormLabel(props.dataEntryForm.name);
            me._calculateAndSetMinWidth(me.domNode);
        },
        createAdminToolbar: function(props) {
            var me = this;

            me.addChild(me._createSaveLayoutButton());
            me.addChild(new ToolbarSeparator());
            me.addChild(me._createCommentsButton(props.commentLinkUrl));
            me._createAndPlaceFormLabel(props.dataEntryForm.name);
            me._calculateAndSetMinWidth(me.domNode);
        },
        createApproverToolbar: function(props) {
            var me = this;
            var submitterItems = me._createSubmitterDropdown(props.users);

            me.addChild(me._createApproveButton());
            me.addChild(me._createRejectButton());
            me.addChild(new ToolbarSeparator());
            me.addChild(me._createCommentsButton(props.commentLinkUrl));
            domConstruct.place(submitterItems.label, me.domNode, 'last');
            me.addChild(submitterItems.dropDown);
            me._createAndPlaceFormLabel(props.dataEntryForm.name);
            me._calculateAndSetMinWidth(me.domNode);
        },
        updateApproverToolbar: function(props) {
            var me = this;

            me._submitterDropdown.set('store', me._createSubmitterStore(props.users));
            me._submitterDropdown.set('value', 'all');
        },
        getSelectedSubmitter: function() {
            var me = this;

            return me._submitterDropdown.store.get(me._submitterDropdown.get('value'));
        },

        //
        // Private functions
        //

        _createSaveDataButton: function() {
            var me = this;
            var saveDataButton = new Button({
                iconClass: 'sasSaveIcon',
                showLabel: false,
                label: resources['de.portlet.form.datagrid.savedata.toolbar.txt']
            });

            me.own(
                saveDataButton.on('click', function () {
                    topic.publish(AppConst.SAVE_FORM_CLICK);
                })
            );

            return saveDataButton;
        },
        _createSaveLayoutButton: function() {
            var me = this;
            var saveLayoutButton = new Button({
                iconClass: 'sasColumnIcon',
                showLabel: false,
                label: resources['de.portlet.form.datagrid.savelayout.toolbar.txt']
            });

            me.own(
                saveLayoutButton.on('click', function() {
                    topic.publish(AppConst.SAVELAYOUT_FORM_CLICK);
                })
            );

            return saveLayoutButton;
        },
        _createSubmitDataButton: function() {
            var me = this;
            var submitDataButton = new Button({
                iconClass: 'sasSubmitFormIcon',
                showLabel: false,
                label: resources['de.portlet.form.datagrid.submitdata.toolbar.txt']
            });

            me.own(
                submitDataButton.on('click', function() {
                    topic.publish(AppConst.SUBMIT_FORM_CLICK);
                })
            );

            return submitDataButton;
        },
        _createCommentsButton: function(commentLinkUrl) {
            var me = this;
            var commentsButton = new Button({
                iconClass: 'sasCommentsBlankIcon',
                showLabel: false,
                label: resources['de.portlet.form.datagrid.comments.toolbar.txt']
            });

            me.own(
                commentsButton.on('click', function() {
                    topic.publish(AppConst.COMMENTS_CLICK, commentLinkUrl);
                })
            );

            return commentsButton;
        },
        _createResetFormButton: function() {
            var me = this;
            var resetFormButton = new Button({
                iconClass: 'sasResetIcon',
                showLabel: false,
                label: resources['de.portlet.form.datagrid.resetform.toolbar.txt']
            });

            me.own(
                resetFormButton.on('click', function() {
                    topic.publish(AppConst.RESET_FORM_CLICK);
                })
            );

            return resetFormButton;
        },
        _createApproveButton: function() {
            var me = this;
            var approveButton = new Button({
                iconClass: 'sasApproveIcon',
                showLabel: false,
                label: resources['de.portlet.form.datagrid.approve.toolbar.txt']
            });

            me.own(
                approveButton.on('click', function() {
                    topic.publish(AppConst.APPROVE_FORM_CLICK);
                })
            );

            return approveButton;
        },
        _createRejectButton: function() {
            var me = this;
            var rejectButton = new Button({
                iconClass: 'sasRejectFormIcon',
                showLabel: false,
                label: resources['de.portlet.form.datagrid.reject.toolbar.txt']
            });

            me.own(
                rejectButton.on('click', function() {
                    topic.publish(AppConst.REJECT_FORM_CLICK);
                })
            );

            return rejectButton;
        },
        _createSubmitterDropdown: function(submitters) {
            var me = this;
            var result = {};
            var dropDown;

            result.label = domConstruct.toDom('<label>' + resources['de.portlet.form.datagrid.submitter.txt'] + '</label>');
            result.dropDown = dropDown = new STMFilteringSelect({
                store: me._createSubmitterStore(submitters),
                required: false,
                value: 'all'
            });

            me._submitterDropdown = dropDown;
            me.own(
                dropDown.on('change', function(newSubmitter) {
                    topic.publish(AppConst.SUBMITTER_CHANGED, newSubmitter);
                })
            );

            return result;
        },
        _createAndPlaceFormLabel: function(formName) {
            var me = this;
            var labelClass = STMUtil.isRtl() ? 'stmTopLeftCorner' : 'stmTopRightCorner';

            domConstruct.place(
                '<label class="' + labelClass + '">' + resources['de.portlet.form.datagrid.formname.txt'] + formName + '</label>',
                me.domNode, 'last');
        },
        _createSubmitterStore: function(submitters) {
            // Mark already approved submitters as disabled
            array.forEach(submitters, function(submitter) {
                if (submitter.approved) {
                    submitter.disabled = true;
                }
            });

            // Add "All" option to submitter list
            submitters.unshift({
                name: resources['all.txt'],
                id: 'all'
            });

            return new Memory({data: submitters});
        },
        // Set the min-width to the computed widths of all of the node's children
        _calculateAndSetMinWidth: function(node) {
            domStyle.set(node, 'minWidth', STMUtil.getComputedWidthOfChildren(node) + 'px');
        }

    });

    return DataEntryToolbarView;

});
